@@ -22,7 +22,7 @@ Copyright 2003-2007 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: data.c 714 2007-05-22 17:06:46Z beerli $
+$Id: data.c 1077 2007-12-19 15:30:49Z beerli $
  
 -------------------------------------------------------*/
 /*! \file data.c
@@ -467,14 +467,15 @@ void read_header (FILE * infile, data_fmt * data, option_fmt * options)
             sscanf (input, "%ld%ld%[^\n]", &(data->numpop), &(data->loci),
                     title);
             find_delimiter (title, &data->dlm);
-	if(!(title[0]==' ' || title[0] == '\0'))
+	    
+	if(!(title[0] == '\0'))
 	  strcpy(options->title,title);
             break;
         case 'b':
         case 'm':
             sscanf (input, "%ld%ld%1s%[^\n]", &(data->numpop), &(data->loci),
                     &(data->dlm), title);
-	    if(!(title[0]==' ' || title[0] == '\0'))
+	    if(!(title[0] == '\0'))
 	      strcpy(options->title,title);
             break;
         case 's':
@@ -483,7 +484,7 @@ void read_header (FILE * infile, data_fmt * data, option_fmt * options)
         case 'f':
             sscanf (input, "%ld%ld%[^\n]", &(data->numpop), &(data->loci),
                     title);
-	    if(!(title[0]==' ' || title[0] == '\0'))
+	    if(!(title[0] == '\0'))
 	      strcpy(options->title,title);
             break;
         default:
@@ -1133,7 +1134,8 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
     total = (long *) mycalloc(data->loci,sizeof(long));
     totalmiss = (long *) mycalloc(data->loci,sizeof(long));
     fprintf (file, "\nSummary of data:\n");
-    fprintf (file, "---------------\n");
+    fprintf(file, "Title: %s\n",options->title);
+
     switch (options->datatype)
     {
     case 'a':
@@ -1552,6 +1554,7 @@ create_alleles (data_fmt * data, option_fmt *options)
 	    if(options->mu_rates==NULL)
 	      {
 		options->mu_rates = (MYREAL * ) mycalloc(data->loci,sizeof(MYREAL));
+		//		printf("%i> data.c: 1557 murate size %li\n",myID,data->loci * sizeof (MYREAL));
 	      }
 	    options->mu_rates[locus] = z;
 	    n = n + 1;
@@ -1681,7 +1684,7 @@ read_geofile (data_fmt * data, option_fmt * options, long numpop)
                 {
                     data->geo[mm2m (i, j, numpop)] =   1. / data->ogeo[i][j];
                     data->lgeo[mm2m (i, j, numpop)] =  data->ogeo[i][j] > 0.0 ?
-                                                       LOG (1. / data->ogeo[i][j]) : -DBL_MAX;
+                                                       LOG (1. / data->ogeo[i][j]) : -MYREAL_MAX;
                 }
             }
         }
@@ -1713,7 +1716,11 @@ read_date_fromfile (FILE * datefile, data_fmt *data, option_fmt *options, long n
 		for(ind=0; ind < data->numind[pop][locus]; ind++)
 		  {
 		    FGETS (input, LINESIZE, datefile);
+#ifdef USE_MYREAL_FLOAT
+		    sscanf (input+nmlength, "%f",&temp);
+#else
 		    sscanf (input+nmlength, "%lf",&temp);
+#endif
 		    fprintf(stdout,"Locus %li: Tipdate %*.*s %f %f\n", locus, (int) nmlength, (int) nmlength, input, temp, temp * options->mutationrate_year[locus] * options->generation_year);
 		    data->sampledates[pop][locus][ind].date = temp;
 		    if(oldest < temp)
