@@ -15,7 +15,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: gammalike.c 514 2006-11-09 20:14:08Z beerli $
+$Id: gammalike.c 886 2007-08-18 03:09:50Z beerli $
 */
 /*! \file gammalike.c 
 
@@ -129,7 +129,7 @@ gamma_loci_like_new_notworking(helper_fmt * helper, MYREAL *oparam, MYREAL *olpa
 {
     long locus=0;
     long trials=0;
-    MYREAL logalpha= -DBL_MAX;
+    MYREAL logalpha= -MYREAL_MAX;
     MYREAL psiabc;
     MYREAL theta1 = helper->nr->param[0];
     nr_fmt *nr = helper->nr;
@@ -233,7 +233,7 @@ gamma_locus_like (nr_fmt * nr, MYREAL *oparam, MYREAL *olparam, MYREAL denom,
 {
     //  nr_fmt *nr = helper->nr;
     long r;
-    MYREAL tempmax = -DBL_MAX;
+    MYREAL tempmax = -MYREAL_MAX;
     MYREAL *temp;
     MYREAL summ = 0.;
     MYREAL alpha = oparam[nr->numpop2];
@@ -342,14 +342,14 @@ gamma_locus_derivative (helper_fmt * helper, long locus)
     temp[0] = (MYREAL *) mycalloc (partsize * GAMMA_INTERVALS, sizeof (MYREAL));
     stemp = (int **) mycalloc (partsize, sizeof (int *));
     stemp[0] = (int *) mycalloc (partsize * GAMMA_INTERVALS, sizeof (int));
-    tempmax[0] = -DBL_MAX;
+    tempmax[0] = -MYREAL_MAX;
     for (r = 1; r < partsize; r++)
     {
         temp[r] = temp[0] + r * GAMMA_INTERVALS;
         stemp[r] = stemp[0] + r * GAMMA_INTERVALS;
-        tempmax[r] = -DBL_MAX;
+        tempmax[r] = -MYREAL_MAX;
     }
-    //summllmax = -DBL_MAX;
+    //summllmax = -MYREAL_MAX;
     //summllsum=0.0;
     for (r = 0; r < GAMMA_INTERVALS; r++)
     {
