@@ -1,4 +1,3 @@
-
 /*------------------------------------------------------
  Maximum likelihood estimation 
  of migration rate  and effectice population size
@@ -17,7 +16,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: lrt.c 514 2006-11-09 20:14:08Z beerli $
+$Id: lrt.c 904 2007-09-11 13:23:02Z beerli $
  
 -------------------------------------------------------*/
 /* \file lrt.c 
@@ -405,7 +404,7 @@ void print_box(world_fmt* world,char **box,long size,
 {
     long i;
     FPRINTF(world->outfile,"%-*.*s | LnL(test) = %f\n",BOXSIZE,BOXSIZE, box[0],like0);
-    FPRINTF(world->outfile,"%-*.*s | LNL(full) = %f\n",BOXSIZE,BOXSIZE, box[1],like1);
+    FPRINTF(world->outfile,"%-*.*s | LnL(full) = %f\n",BOXSIZE,BOXSIZE, box[1],like1);
     FPRINTF(world->outfile,"%-*.*s | LRT       = %f\n",BOXSIZE,BOXSIZE, box[2],lrt);
     FPRINTF(world->outfile,"%-*.*s | df        = %li\n",BOXSIZE,BOXSIZE,box[3],(long) df);
     FPRINTF(world->outfile,"%-*.*s | Prob      = %f\n",BOXSIZE,BOXSIZE,box[4],chiprob);
@@ -417,7 +416,7 @@ void print_box(world_fmt* world,char **box,long size,
     if(world->options->progress)
     {
         FPRINTF(stdout,"%-*.*s | LnL(test) = %f\n",BOXSIZE,BOXSIZE, box[0],like0);
-        FPRINTF(stdout,"%-*.*s | LNL(full) = %f\n",BOXSIZE,BOXSIZE, box[1],like1);
+        FPRINTF(stdout,"%-*.*s | LnL(full) = %f\n",BOXSIZE,BOXSIZE, box[1],like1);
         FPRINTF(stdout,"%-*.*s | LRT       = %f\n",BOXSIZE,BOXSIZE, box[2],lrt);
         FPRINTF(stdout,"%-*.*s | df        = %li\n",BOXSIZE,BOXSIZE,box[3],(long) df);
         FPRINTF(stdout,"%-*.*s | Prob      = %f\n",BOXSIZE,BOXSIZE,box[4],chiprob);
