@@ -26,7 +26,7 @@
 *   works and am not responsible for any damage you may cause or have.
 *
 */
-/* $Id: main.c 1102 2008-01-01 23:45:34Z beerli $*/
+/* $Id: main.c 1121 2008-01-14 14:00:46Z beerli $*/
 
 #include "migration.h"
 #include "sighandler.h"
@@ -463,7 +463,7 @@ main (int argc, char **argv)
     }
     else
     {
-      if(EARTH->options->bayes_infer)
+      if(options->bayes_infer)
 	{
 	  //	  analyze_oldbayesdata(EARTH, options, data, &outfilepos);
 	}
@@ -547,7 +547,9 @@ main (int argc, char **argv)
 	if(options->mighist || options->skyline)
 	  {
 	    print_event_values(EARTH);
+#ifdef PRETTY
 	    pdf_histogram_legend();
+#endif
 	  }        
 
 
@@ -2326,6 +2328,7 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
   long maxreplicate = (options->replicate
 		       && options->replicatenum >
 		       0) ? options->replicatenum : 1;
+  //  long lsteps = options->lsteps;
   //  MYREAL sum = 0.;
   MYREAL heat0 = 1.;
   MYREAL heat1 = 1.;
@@ -2333,7 +2336,7 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
   MYREAL hsum = 0.;
   MYREAL asum = 0.;
   MYREAL lsum;
-  fprintf(file,"\n\n\nLog-Probability of the data given the model (marginal likelihood)\n");
+  fprintf(file,"\n\n\nLog-Probability of the data given the model (marginal likelihood = log(P(D|thisModel))\n");
   fprintf(file,"--------------------------------------------------------------------\n[Use this value for Bayes factor calculations:\nBF = Exp[log(P(D|thisModel) - log(P(D|otherModel)]\nshows the support for thisModel]\n\n");
   
   if(options->heating)
@@ -2361,12 +2364,15 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 		  heat1 = universe[t]->heat;
 		}
 #endif
-	      lsum += (heat0 - heat1) * ((EARTH->bf[locus * hc + t-1] + EARTH->bf[locus * hc + t])/(2. * EARTH->options->lsteps*maxreplicate));
-	      //	  printf("%i> sum[%li]=%f temp=(%f %f) values=(%f %f) scale=%f\n",myID,t,sum,heat0,heat1,log(EARTH->bf[t-1]),log(EARTH->bf[t]), EARTH->bfscale);
+	      lsum += (heat0 - heat1) * ((EARTH->bf[locus * hc + t-1] + EARTH->bf[locus * hc + t]) * 0.5);
+	      printf("%i> sum[%li]=%f temp=(%f %f) values=(%f %f)\n",myID,t,lsum,heat0,heat1,EARTH->bf[locus * hc + t-1],EARTH->bf[locus * hc + t]);
 	    }
-	  bfsum += log(lsum) + EARTH->bfscale[locus];
+	  //	  fprintf(stdout,"Thermo[%li]=%f\n",locus, log(lsum) + EARTH->bfscale[locus]);
+	  fprintf(stdout,"Thermo[%li]=%f\n",locus, lsum);
+	  //	  bfsum += log(lsum) + EARTH->bfscale[locus];
+	  bfsum += lsum; //+ EARTH->bfscale[locus];
 	}
-      fprintf(file,"(1) Thermodynamic integration: log(Prob(D|Model))=%f %s\n", bfsum, 
+      fprintf(file,"(1) Thermodynamic integration: log(Prob(D|Model))=%f %s\n", bfsum,  
       	      options->adaptiveheat ? "(adaptive heating -> value may be wrong)" : "");
     }
   else
@@ -2382,36 +2388,37 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
       //printf(" %f %f\n",EARTH->hm[locus],EARTH->am[locus]);
       if(EARTH->data->skiploci[locus])
 	continue;
-      hsum += EARTH->hmscale[locus] - log (EARTH->hm[locus] / (EARTH->options->lsteps*maxreplicate));
+      fprintf(stdout,"    Harmonic[%li]=%f\n",locus, EARTH->hmscale[locus] - log (EARTH->hm[locus]));
+      hsum +=  EARTH->hmscale[locus] - log (EARTH->hm[locus]);
     }
   fprintf(file,"(2) Harmonic mean:             log(Prob(D|Model))=%f\n",hsum);
-      asum =  0.0;
-      //      for(locus=0;locus < EARTH->loci; locus++)
-      //	{
-      //	  if(EARTH->data->skiploci[locus])
-      //	    continue;
-      //	  asum += EARTH->amscale[locus]  + log (EARTH->am[locus] / (EARTH->options->lsteps*maxreplicate));
-      //	}
-      //      fprintf(file,"(3) Arithmetic mean:           log(Prob(D|Model))=%f\n", asum);
-      fprintf(file,"(1) and (2) should give a similar result, (2) is considered more\ncrude than (1), but (1) needs heating with several well-spaced chains\n\n");
-      //(3) maybe strongly biased by large values\n\n\n");
+  asum =  0.0;
+  //      for(locus=0;locus < EARTH->loci; locus++)
+  //	{
+  //	  if(EARTH->data->skiploci[locus])
+  //	    continue;
+  //	  asum += EARTH->amscale[locus]  + log (EARTH->am[locus] / (lsteps*maxreplicate));
+  //	}
+  //      fprintf(file,"(3) Arithmetic mean:           log(Prob(D|Model))=%f\n", asum);
+  fprintf(file,"(1) and (2) should give a similar result, (2) is considered more\ncrude than (1), but (1) needs heating with several well-spaced chains\n\n");
+  //(3) maybe strongly biased by large values\n\n\n");
 #ifdef PRETTY
-      if(file==EARTH->outfile)
-	pdf_bayes_factor(EARTH, bfsum, hsum, asum, maxreplicate);
+  if(file==EARTH->outfile)
+    pdf_bayes_factor(EARTH, bfsum, hsum, asum, maxreplicate);
 #endif
 } 
-
-/// calculate bayes factor using thermodynamic integration
+  
+/// calculate values for the marginal likelihood using thermodynamic integration
 /// based on a method by Friel and Pettitt 2005
 /// (http://www.stats.gla.ac.uk/research/TechRep2005/05.10.pdf)
 /// this is the same method described in Lartillot and Phillippe 2006 Syst Bio
 /// integrate over all temperature using a simple trapezoidal rule
 /// prob(D|model) is only accurate with intervals for temperatures from 1/0 to 1/1.
-/// reports also the harmonic mean and the arithmetic mean
+/// reports also the harmonic mean
 void calculate_BF(world_fmt **universe, option_fmt *options)
 {
   long i;
-  MYREAL xx;
+  MYREAL xx, xx2;
   //  MYREAL logprior;
   long locus = universe[0]->locus;
   long hc = options->heated_chains;
@@ -2419,40 +2426,43 @@ void calculate_BF(world_fmt **universe, option_fmt *options)
     return;
   if(EARTH->likelihood[EARTH->G] <= -HUGE)
     return;
+  EARTH->am[locus] += 1;
+  locus = EARTH->locus;
   if(options->heating)
     {
       for (i = 0; i < hc; i++)
 	{
-	  locus = universe[i]->locus;
 	  xx = universe[i]->likelihood[universe[i]->G];
 #ifdef THERMOCHECK
-	  fprintf(stderr,"@@@ %10.10f %10.10f\n",universe[i]->heat, xx);
+	  fprintf(stderr,"\n@@@T %li %10.10f %10.10f ",locus, universe[i]->heat, xx);
+#endif
+	  EARTH->bf[locus * hc + i] += (xx - EARTH->bf[locus * hc + i])/ (EARTH->am[locus]);
+#ifdef THERMOCHECK
+	  fprintf(stderr,"%f\n", 	  EARTH->bf[locus * hc + i]);
 #endif
-	  if(xx < EARTH->bfscale[locus])
-	    {
-	      EARTH->bf[locus * hc + i] += EXP(xx - EARTH->bfscale[locus]);
-	    }
-	  else
-	    {
-	      EARTH->bf[locus * hc + i] *= EXP(EARTH->bfscale[locus] - xx);
-	      EARTH->bfscale[locus] = xx;
-	      EARTH->bf[locus * hc + i] += 1.0; //exp(universe[i]->param_like - universe[i]->bfscale);
-	    }
 	}
     }
   xx = EARTH->likelihood[EARTH->G];
   if(xx <= -HUGE)
     return;
-  locus = EARTH->locus;
+
+#ifdef THERMOCHECK
+  fprintf(stderr,"\n@@@H %li %10.10f %10.10f %10.10f\n",locus, EARTH->heat, xx, EARTH->hmscale[locus]);
+#endif
+  
   if(xx > EARTH->hmscale[locus])
     {
-        EARTH->hm[locus] += EXP(EARTH->hmscale[locus] - xx);
+      //      EARTH->hm[locus] += EXP(EARTH->hmscale[locus] - xx);
+      xx2 = EXP(EARTH->hmscale[locus] - xx);
+      EARTH->hm[locus] += (xx2 - EARTH->hm[locus])/ (EARTH->am[locus]);
     }
   else
     {
       EARTH->hm[locus] *= EXP(xx - EARTH->hmscale[locus]);
+      //      EARTH->hm[locus] *= xx - EARTH->hmscale[locus];
       EARTH->hmscale[locus] = xx;
-      EARTH->hm[locus] += 1.;
+      //      EARTH->hm[locus] += 1.;
+      EARTH->hm[locus] += (1. - EARTH->hm[locus])/ (EARTH->am[locus]);
     }
 }
 
