@@ -18,7 +18,7 @@ Copyright 2003-2006 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: mcmc1.c 733 2007-05-31 02:28:34Z beerli $
+$Id: mcmc1.c 1068 2007-12-14 19:10:22Z beerli $
 -------------------------------------------------------*/
 /* \file mcmc1.c
 
@@ -73,8 +73,8 @@ void findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
                       long gte);
 void free_proposal (proposal_fmt * proposal);
 void free_timevector (timelist_fmt * timevector);
-long prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv, node ** ptr, long numpop);
-long remove_doublettes_old (timelist_fmt * timevector, node ** ptr);
+void prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv, register node ** __restrict ptr, proposal_fmt * proposal, long numpop);
+
 long xor (node ** ptrl1, node ** ptrl2);
 long rmigrcount (proposal_fmt * proposal);
 
@@ -180,12 +180,6 @@ tree_update (world_fmt * world, long g)
     //
     construct_localtimelist (timevector, proposal);
     tentry = &(*timevector).tl[0];
-    //    if (proposal->origin->type == 't')
-    //{
-    //    age = proposal->origin->tyme;
-    //}
-    //else
-    //{
     age = proposal->origin->tyme;
     zz = 0;
     while (tentry->age <= age && zz < (*timevector).T)
@@ -193,7 +187,6 @@ tree_update (world_fmt * world, long g)
 	tentry = &(*timevector).tl[zz];
 	zz++;
       }
-    //}
     nexttime = tentry->age;
     if ((*timevector).T > 1)
       endtime = (*timevector).tl[(*timevector).T - 2].age;
@@ -262,8 +255,7 @@ tree_update (world_fmt * world, long g)
     }
     if (!coalesced)
     {
-      if (!pre_population
-	  (proposal, (*timevector).tl, (*timevector).T - 1, &slider))
+      if (!pre_population(proposal, (*timevector).tl, (*timevector).T - 1, &slider))
         {
 	  free_proposal (proposal);
 	  free_timevector (timevector);
@@ -371,12 +363,10 @@ new_localtimelist (timelist_fmt ** ntl, timelist_fmt * otl, long numpop)
     (*ntl)->tl = (vtlist *) mymalloc ((*otl).allocT * sizeof (vtlist));
     (*ntl)->allocT = otl->allocT;
     (*ntl)->T = otl->T;
-    // not needed with direct approach without sort 
-    memcpy ((*ntl)->tl, otl->tl, otl->allocT * sizeof (vtlist));
-    allocate_lineages (ntl,(*ntl)->tl, (*ntl)->allocT,
-                       (*ntl)->T, numpop);
-    // not needed with direct approach without sort 
-    memcpy ((*ntl)->lineages, otl->lineages, (*ntl)->allocT * numpop * sizeof (long));
+    (*ntl)->oldT = otl->oldT;
+    //memcpy ((*ntl)->tl, otl->tl, otl->allocT * sizeof (vtlist));
+    allocate_lineages (ntl, 0, numpop);
+    //memcpy ((*ntl)->lineages, otl->lineages, (*ntl)->allocT * numpop * sizeof (long));
 }
 
 ///
@@ -396,8 +386,7 @@ new_localtimelist_new (timelist_fmt ** ntl, timelist_fmt * otl, long numpop)
     }
   (*ntl)->T = otl->T;
   memcpy ((*ntl)->tl, otl->tl, otl->allocT * sizeof (vtlist));
-  allocate_lineages (ntl,(*ntl)->tl, (*ntl)->allocT,
-                       (*ntl)->T, numpop);
+  allocate_lineages (ntl, 0, numpop);
     memcpy ((*ntl)->lineages, otl->lineages, (*ntl)->allocT * numpop * sizeof (long));
 }
 
@@ -641,6 +630,33 @@ void traverse_check(node *theNode)
     }
 }
 
+///
+/// tags the lineage that was added the last time the tree was changed.
+void traverse_tagnew(node *theNode, node *origin)
+{
+  if (theNode == NULL)
+    error("a missing node pointer encountered, aborted");
+  theNode->visited = FALSE;
+  if (theNode->type != 't'  && theNode->next->next->back != NULL)
+    {
+      if(theNode->top != 1)
+	error("");
+      traverse_tagnew (theNode->next->back, origin);       
+      if (theNode->type != 'm' && theNode->next->next->back != NULL)
+	{
+	  traverse_tagnew (theNode->next->next->back, origin);
+	}
+    }
+  if(theNode==origin)
+    {
+      while(theNode != NULL && theNode->type != 'r')
+	{
+	  theNode = showtop(showtop(theNode)->back);
+	  theNode->visited = TRUE;
+	}
+    }
+}
+
 ///
 /// pick a node at random from the available list of nodes, this code will even 
 /// work with datasets where there are only two nodes.
@@ -685,6 +701,10 @@ chooseOrigin (proposal_fmt * proposal)
         error ("Designation of origin for branch removal failed");
 }
 
+
+///
+/// construct a list of ordered times with pointers to the tree
+/// using the origin to prune the old time list
 void
 construct_localtimelist (timelist_fmt * timevector, proposal_fmt * proposal)
 {
@@ -695,17 +715,7 @@ construct_localtimelist (timelist_fmt * timevector, proposal_fmt * proposal)
                       &proposal->aboveorigin, &z, &oz, WITH_MIGR_NODES);
     proposal->aboveorigin[z++] = proposal->oback;
     proposal->aboveorigin[z] = NULL;
-    //z = remove_doublettes_old (timevector, proposal->aboveorigin);
-    z = prune_timelist(&proposal->world->treetimes[0],timevector, proposal->aboveorigin, proposal->world->numpop);
-    // not needed with direct approach without sort 
-    //qsort ((void *) (*timevector).tl, (*timevector).T, sizeof (vtlist), agecmp);
-    //(*timevector).T -= z;
-    if ((*timevector).tl[(*timevector).T - 1].eventnode->type != 'r')
-    {
-        error ("Root not at the end of timelist\n");
-    }
-    timeslices (&timevector);
-    add_locallineages (timevector, proposal);
+    prune_timelist(&proposal->world->treetimes[0],timevector, proposal->aboveorigin, proposal, proposal->world->numpop);
 }
 
 /*----------------------------------------------------------------------------
@@ -730,88 +740,86 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
       }
     
     if (include_migration == NO_MIGR_NODES)
-    {
-      // nodelist needs to be at least twice as long as sumtips
-      // otherwise it will run out of reserved memory 
+      {
+	// nodelist needs to be at least twice as long as sumtips
+	// otherwise it will run out of reserved memory 
         if (theNode->type != 't')
-        {
+	  {
             if (crawlback (theNode->next) != NULL)
-                traverseAllNodes (crawlback (theNode->next), nodelist, node_elem,
-                                  oldnode_elem, NO_MIGR_NODES);
+	      traverseAllNodes (crawlback (theNode->next), nodelist, node_elem,
+				oldnode_elem, NO_MIGR_NODES);
             if (theNode->type != 'm' && crawlback (theNode->next->next) != NULL)
-                traverseAllNodes (crawlback (theNode->next->next), nodelist,
-                                  node_elem, oldnode_elem, NO_MIGR_NODES);
+	      traverseAllNodes (crawlback (theNode->next->next), nodelist,
+				node_elem, oldnode_elem, NO_MIGR_NODES);
             (*nodelist)[(*node_elem)] = theNode;
             (*node_elem) += 1;
             if (theNode->type == 'm')
-            {
+	      {
                 error ("Migration node encountered?! and died!");
-            }
+	      }
 	    if(*node_elem > *oldnode_elem)
 	      error("die on the spot, this should not happen. Failed in traverseAllNodes without migration nodes");
-        }
+	  }
         else
-        {
-	  (*nodelist)[(*node_elem)] = theNode;
-	  (*node_elem) += 1;
-	  if(*node_elem > *oldnode_elem)
-	    error("die on the spot, this should not happen. Failed in traverseAllNodes without migration nodes at tip");
-        }
-    }
+	  {
+	    (*nodelist)[(*node_elem)] = theNode;
+	    (*node_elem) += 1;
+	    if(*node_elem > *oldnode_elem)
+	      error("die on the spot, this should not happen. Failed in traverseAllNodes without migration nodes at tip");
+	  }
+      }
     else
-    {
+      {
         if (theNode->type != 't')
-        {
+	  {
             if (theNode->next->back != NULL)
-                traverseAllNodes (theNode->next->back, nodelist, node_elem,
-                                  oldnode_elem, WITH_MIGR_NODES);
+	      traverseAllNodes (theNode->next->back, nodelist, node_elem,
+				oldnode_elem, WITH_MIGR_NODES);
             if (theNode->type != 'm' && theNode->next->next->back != NULL)
-                traverseAllNodes (theNode->next->next->back, nodelist, node_elem,
-                                  oldnode_elem, WITH_MIGR_NODES);
+	      traverseAllNodes (theNode->next->next->back, nodelist, node_elem,
+				oldnode_elem, WITH_MIGR_NODES);
             if ((*node_elem) == (*oldnode_elem - 2))
-            {
+	      {
                 elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
                 (*nodelist) =
-                    (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
+		  (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
                 memset ((*nodelist) + (*oldnode_elem), 0,
                         sizeof (node *) * (elem - (*oldnode_elem)));
                 *oldnode_elem = elem;
-            }
+	      }
             (*nodelist)[(*node_elem)++] = theNode;
-        }
+	  }
         else
-        {
+	  {
             if ((*node_elem) == (*oldnode_elem - 2))
-            {
+	      {
                 elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
                 (*nodelist) =
-                    (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
+		  (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
                 memset ((*nodelist) + (*oldnode_elem), 0,
                         sizeof (node *) * (elem - (*oldnode_elem)));
                 *oldnode_elem = elem;
-            }
+	      }
             (*nodelist)[(*node_elem)++] = theNode;
-        }
-    }
+	  }
+      }
 }
 
 ///
 /// copies elements from the old timelist (oldtv)
 /// into the new local timeslist (newtv) checking whether they
 /// are used after the removal of the residual tree that is above the origin (ptr)
-long
-prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv, node ** ptr, long numpop)
+void
+prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv,  register node ** __restrict ptr, proposal_fmt * proposal, long numpop)
 {
   long i = 0;
-  long j = 0;
+  register long j = 0;
   long slot = 0;
-  long size = numpop * sizeof(long);
-  //  vtlist *otl = (*oldtv).tl;
-  //vtlist *ntl = (*newtv).tl;
-  long * tmpptr = NULL;
-  node *thenode;
-  //printf("oldtv.T=%li newtv.T=%li ", (*oldtv).T, (*newtv).T);
-    /* assumes that there is an NULL element at the end */
+  //  long size = numpop * sizeof(long);
+  //  long * tmpptr = NULL;
+  register node *thenode;
+  vtlist * tls;
+
     slot=0;
     for (i = 0; i < (*oldtv).T; i++)
     {
@@ -819,48 +827,26 @@ prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv, node ** ptr, long nu
       thenode = (*oldtv).tl[i].eventnode;
       while ((thenode != ptr[j]) && (ptr[j] != NULL))
 	{
-	  //	  printf("%5li %5li old=%p ptr=%p\n",i, j ,(*oldtv).tl[i].eventnode,ptr[j]);
 	  j++;
 	}
-      //      printf("*** %5li %5li old=%p ptr=%p\n",i, j ,(*oldtv).tl[i].eventnode,ptr[j]);
       if (ptr[j] == NULL)
 	{
-	  tmpptr = (*newtv).tl[slot].lineages;
-	  memcpy(&(*newtv).tl[slot],&(*oldtv).tl[i],sizeof(vtlist));
-	  (*newtv).tl[slot].lineages = tmpptr;
-	  memcpy((*newtv).tl[slot].lineages,(*oldtv).tl[i].lineages,size);
-	  //printf("in tree %li %li (%li)\n",z,i,otl[i].eventnode->number);
+	  tls = &((*newtv).tl[slot]);
+	  tls->eventnode = thenode ;
+	  tls->age       = (*oldtv).tl[i].age ;
+	  tls->interval  = (*oldtv).tl[i].interval ;
+	  tls->from      = thenode->pop;
+	  tls->to        = thenode->actualpop;
+	  tls->slice     = slot;
 	  slot++;
 	}
-      //      else
-      //	{
-	  //printf("excised from timelist %li %li\n", i, otl[i].eventnode->number);
-      //	  (*newtv).T -= 1;
-      //	}
     }
     (*newtv).T = slot;
-    //printf("oldtv.T=%li newtv.T=%li last type=%c\n", (*oldtv).T, (*newtv).T, (*newtv).tl[(*newtv).T-1].eventnode->type);
-    return (*oldtv).T - slot;
-}
-
-/* prepares to remove elements of a timelist by setting eventnode to NULL
-   and age to Infinity, so a simple sort will push them over the edge */
-long
-remove_doublettes_old(timelist_fmt * timevector, node ** ptr)
-{
-    long i = 0, j = 0, slot = 0;
-    /* assumes that there is an NULL element at the end */
-    for (i = 0; i < (*timevector).T; j = 0, i++)
+    if ((*newtv).tl[(*newtv).T - 1].eventnode->type != 'r')
     {
-        while (((*timevector).tl[i].eventnode != ptr[j]) && (ptr[j] != NULL))
-            j++;
-        if (ptr[j] != NULL)
-        {
-            slot++;
-            (*timevector).tl[i].age = DBL_MAX;
-        }
+        error ("Root not at the end of local timelist\n");
     }
-    return slot;
+    add_locallineages (newtv, proposal);
 }
 
 void
@@ -1532,6 +1518,8 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
     long rmc = rmigrcount (proposal);
     //zz = world->heat;
     rm =  proposal->migr_table_counter + proposal->migr_table_counter2 + world->migration_counts - rmc;
+    //if(world->heat < 0.001)
+    //  fprintf(stdout,"%i> heat=%f: %li = %li + %li + %li - %li | %li/%li : %li\n",myID, 1./world->heat, rm, proposal->migr_table_counter , proposal->migr_table_counter2 , world->migration_counts , rmc, proposal->world->node_collection_count, proposal->world->node_collection_allocated, proposal->world->node_collection_count - proposal->migr_table_counter - proposal->migr_table_counter2);
     if (rm > limit)
     {
       // warning might disturb parallel runs and might be not really useful at all
