@@ -18,7 +18,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: mcmc2.c 733 2007-05-31 02:28:34Z beerli $
+$Id: mcmc2.c 1089 2007-12-21 22:29:51Z beerli $
 -------------------------------------------------------*/
 /* \file mcmc2.c
 Contains low-level routines to manipulate the treechanger
@@ -29,6 +29,7 @@ Contains low-level routines to manipulate the treechanger
 #include "tree.h"
 #include "uep.h"
 #include "tools.h"
+#include "mcmc.h"
 #include "migrate_mpi.h"
 
 #ifdef DMALLOC_FUNC_CHECK
@@ -70,14 +71,14 @@ void t_s_upper (proposal_fmt * proposal, node * connect, node * obb);
 node *t_s_obbncon (proposal_fmt * proposal, node * obb);
 void t_s_tncon (proposal_fmt * proposal, node * obb);
 void t_s_tcon (proposal_fmt * proposal, node * uppern);
-long erase_migr_nodes (node * up);
+long erase_migr_nodes (world_fmt *world, node * up);
 node *findcrossing (node ** ptrl1, node ** ptrl2);
-void connectnodes (node * mother, node * brother, node * sister);
+void connectnodes (world_fmt *world, node * mother, node * brother, node * sister);
 void gotoroot (node * origin, node ** ptrlist);
 void adjust (node * theNode, MYREAL tyme, long level);
 void localevaluate (node * mother);
 void copy_x (proposal_fmt * proposal, xarray_fmt xx1, xarray_fmt xx2);
-void fix_root_pop (node * p);
+void fix_root_pop (world_fmt *world, node * p);
 void pseudoevaluate (proposal_fmt * proposal, xarray_fmt x, MYREAL *lx,
                      node * mother, node * newdaughter, MYREAL v);
 node *crawl_down (node * theNode, MYREAL tyme);
@@ -88,11 +89,14 @@ node *crawl_down (node * theNode, MYREAL tyme);
 void
 coalesce1p (proposal_fmt * proposal)
 {
+#ifdef MESS
+  int skip=1;
+#endif
+  //static long count=0;
     node *obb = NULL;
-    //traverse_check(crawlback (proposal->root->next));
-    erase_migr_nodes (proposal->origin);
+    erase_migr_nodes (proposal->world, proposal->origin);
     if (proposal->migr_table_counter2 > 0) //|| proposal->mig_removed)
-        erase_migr_nodes (proposal->realtarget);
+      erase_migr_nodes (proposal->world, proposal->realtarget);
     if (proposal->target == proposal->ocousin)
     {
         coat_ocousin (proposal);
@@ -158,6 +162,12 @@ coalesce1p (proposal_fmt * proposal)
     set_pop (proposal->oback, proposal->origin->actualpop,
              proposal->origin->actualpop);
   //  free_migr_node(proposal->origin, crawlback(proposal->origin));
+    //    if(++count % 10000 == 0)
+    //  {
+    //	printf("%li heat %f> %li %li %li %li\n", count, proposal->world->heat, proposal->migr_table_counter, proposal->migr_table_counter2, proposal->world->node_collection_count, proposal->world->node_collection_allocated);
+    //
+    //  }
+
     if (proposal->migr_table_counter > 0)
     {
         set_pop (proposal->oback,
@@ -168,7 +178,7 @@ coalesce1p (proposal_fmt * proposal)
                           &(proposal->migr_table_counter));
     }
     proposal->migr_table_counter = 0;
-    if (proposal->migr_table_counter2)
+    if (proposal->migr_table_counter2 > 0)
     {
         insert_migr_node (proposal->world, proposal->realtarget,
                           crawlback (proposal->target), proposal->migr_table2,
@@ -176,8 +186,21 @@ coalesce1p (proposal_fmt * proposal)
         proposal->migr_table_counter2 = 0;
     }
     /*    if (proposal->oback == crawlback(proposal->root->next)) */
-    fix_root_pop (crawlback (proposal->root->next));
-    //traverse_check(crawlback (proposal->root->next));
+    fix_root_pop (proposal->world, crawlback (proposal->root->next));
+    //    if(proposal->world->in_last_chain)
+    traverse_tagnew(crawlback (proposal->root->next),proposal->origin);
+#ifdef MESS
+    traverse_checker(proposal->root);
+    if(skip==0)
+      {
+	FPRINTF (stdout,
+		 "\n[& Comment: Locus %li, best log likelihood = %f (to file),heat=%f]\n",
+		 proposal->world->locus + 1, proposal->world->likelihood[proposal->world->G], proposal->world->heat);
+	
+	debugtreeout (stdout, crawlback (proposal->world->root->next),
+		      crawlback (proposal->world->root->next), 0);
+      }
+#endif
 }
 
 
@@ -226,19 +249,28 @@ void
 coat_oback (proposal_fmt * proposal)
 {
     node *obb, *obm;
+#ifdef TREEDEBUG
+    printf("coat_oback\n");
+#endif
     obb = showtop (crawlback (proposal->oback));
-    connectnodes (showtop (proposal->oback->back), proposal->osister, NULL);
+    connectnodes (proposal->world,showtop (proposal->oback->back), proposal->osister, NULL);
     adjust_time (proposal->oback, proposal->time);
     obm = showtop (crawl_down (proposal->osister, proposal->time)->back);
-    connectnodes (proposal->oback, proposal->origin, proposal->osister);
+    connectnodes (proposal->world, proposal->oback, proposal->origin, proposal->osister);
     proposal->oback->back = NULL;
+    if (obm->type == 'm')
+      {
+	connectnodes (proposal->world,obm, proposal->oback, NULL);
+      }
+    
     if (obb->type == 'r')
-        connectnodes (obb, proposal->oback, NULL);
+      {
+        connectnodes (proposal->world,obb, proposal->oback, NULL);
+	erase_migr_nodes(proposal->world,proposal->oback);
+      }
     else
     {
-        if (obm->type == 'm')
-            connectnodes (obm, proposal->oback, NULL);
-        connectnodes (obb, proposal->oback, proposal->ocousin);
+      connectnodes (proposal->world,obb, proposal->oback, proposal->ocousin);
     }
     adjust (obb, obb->tyme, 2);
     set_dirty (proposal->oback);
@@ -249,14 +281,18 @@ void
 coat_ocousin (proposal_fmt * proposal)
 {
     node *obb, *rcback;
+#ifdef TREEDEBUG
+    printf("coat_ocousin\n");
+#endif
+
     obb = showtop (crawlback (proposal->oback));
-    connectnodes (showtop (proposal->oback->back), proposal->osister, NULL);
+    connectnodes (proposal->world,showtop (proposal->oback->back), proposal->osister, NULL);
     rcback = showtop (crawl_down (proposal->ocousin, proposal->time)->back);
     adjust_time (proposal->oback, proposal->time);
     proposal->oback->back = NULL;
-    connectnodes (rcback, proposal->oback, NULL);
-    connectnodes (proposal->oback, proposal->ocousin, proposal->origin);
-    connectnodes (obb, proposal->oback, proposal->osister);
+    connectnodes (proposal->world,rcback, proposal->oback, NULL);
+    connectnodes (proposal->world,proposal->oback, proposal->ocousin, proposal->origin);
+    connectnodes (proposal->world,obb, proposal->oback, proposal->osister);
     adjust (obb, obb->tyme, 2);
     set_dirty (proposal->oback);
     set_dirty (obb);
@@ -268,17 +304,21 @@ void
 coat_rbcoa (proposal_fmt * proposal)
 {
     node *obb, *root;
+#ifdef TREEDEBUG
+    printf("coat_rbcoa\n");
+#endif
+
     if (proposal->ocousin != NULL)
     {
         obb = showtop (crawlback (proposal->oback));
         root = proposal->root;
-        connectnodes (showtop (proposal->oback->back), proposal->osister, NULL);
-        connectnodes (obb, proposal->ocousin, proposal->osister);
+        connectnodes (proposal->world,showtop (proposal->oback->back), proposal->osister, NULL);
+        connectnodes (proposal->world,obb, proposal->ocousin, proposal->osister);
         adjust (obb, obb->tyme, 2);
         adjust_time (proposal->oback, proposal->time);
         proposal->oback->back = NULL;
-        connectnodes (proposal->oback, proposal->origin, proposal->target);
-        connectnodes (root, proposal->oback, NULL);
+        connectnodes (proposal->world,proposal->oback, proposal->origin, proposal->target);
+        connectnodes (proposal->world,root, proposal->oback, NULL);
         adjust (proposal->oback, proposal->time, 2);
         set_dirty (obb);
         set_dirty (proposal->oback);
@@ -294,23 +334,28 @@ void
 coat_obbcoa (proposal_fmt * proposal)
 {
     node /**obb,*/  * proposalack;
+#ifdef TREEDEBUG
+    printf("coat_obbcoa\n");
+#endif
+
     if (proposal->ocousin != NULL)
     {
         /*   obb = showtop(crawlback(proposal->oback)); */
-        connectnodes (showtop (proposal->oback->back), proposal->osister, NULL);
-        proposalack = showtop (crawlback (proposal->target));
+        connectnodes (proposal->world,showtop (proposal->oback->back), proposal->osister, NULL);
+        // proposalack = showtop (crawl_down (proposal->target, proposal->time)->back);
+	       proposalack = showtop (crawlback (proposal->target));
         adjust_time (proposal->oback, proposal->time);
-        connectnodes (proposal->target, proposal->ocousin, proposal->osister);
+        connectnodes (proposal->world,proposal->target, proposal->ocousin, proposal->osister);
         adjust (proposal->target, proposal->target->tyme, 1);
         proposal->oback->back = NULL;
         adjust_time (proposal->oback, proposal->time);
         if (showtop (proposal->realtarget->back)->type == 'm')
         {
-            connectnodes (showtop (proposal->realtarget->back), proposal->oback,
+            connectnodes (proposal->world,showtop (proposal->realtarget->back), proposal->oback,
                           NULL);
         }
-        connectnodes (proposal->oback, proposal->origin, proposal->target);
-        connectnodes (proposalack, proposal->tsister, proposal->oback);
+        connectnodes (proposal->world,proposal->oback, proposal->origin, proposal->target);
+        connectnodes (proposal->world,proposalack, proposal->tsister, proposal->oback);
         adjust (proposalack, proposalack->tyme, 2);
         set_dirty (proposal->target);
         set_dirty (proposal->oback);
@@ -326,19 +371,26 @@ void
 coat_stancoa (proposal_fmt * proposal)
 {
     node *obb, *proposalack;
+#ifdef TREEDEBUG
+    printf("coat_stancoa\n");
+#endif
+
     if (proposal->ocousin != NULL)
     {
         obb = showtop (crawlback (proposal->oback));
-        proposalack = showtop (crawlback (proposal->target));
-        connectnodes (showtop (proposal->oback->back), proposal->osister, NULL);
+        //proposalack = showtop (crawl_down (proposal->target, proposal->time)->back);
+	proposalack = showtop (crawlback (proposal->target));
+        connectnodes (proposal->world,showtop (proposal->oback->back), proposal->osister, NULL);
         proposal->oback->back = NULL;
-        connectnodes (obb, proposal->osister, proposal->ocousin);
+        connectnodes (proposal->world,obb, proposal->osister, proposal->ocousin);
         adjust_time (proposal->oback, proposal->time);
         if (showtop (proposal->realtarget->back)->type == 'm')
-            connectnodes (showtop (proposal->realtarget->back), proposal->oback,
+	  {
+            connectnodes (proposal->world,showtop (proposal->realtarget->back), proposal->oback,
                           NULL);
-        connectnodes (proposal->oback, proposal->origin, proposal->target);
-        connectnodes (proposalack, proposal->oback, proposal->tsister);
+	  }
+        connectnodes (proposal->world,proposal->oback, proposal->origin, proposal->target);
+        connectnodes (proposal->world,proposalack, proposal->oback, proposal->tsister);
         adjust (proposalack, proposalack->tyme, 3);
         adjust (obb, obb->tyme, 3);
         set_dirty (obb);
@@ -350,18 +402,19 @@ coat_stancoa (proposal_fmt * proposal)
     {
         adjust_time (proposal->oback, proposal->time);
         obb = showtop (crawlback (proposal->oback));
-        connectnodes (showtop (proposal->oback->back), proposal->osister, NULL);
-        proposalack = showtop (crawlback (proposal->target));
+        connectnodes (proposal->world,showtop (proposal->oback->back), proposal->osister, NULL);
+        //proposalack = showtop (crawl_down (proposal->target, proposal->time)->back);
+	       proposalack = showtop (crawlback (proposal->target));
         adjust_time (proposal->oback, proposal->time);
         proposal->oback->back = NULL;
         if (showtop (proposal->realtarget->back)->type == 'm')
-            connectnodes (showtop (proposal->realtarget->back), proposal->oback,
+            connectnodes (proposal->world,showtop (proposal->realtarget->back), proposal->oback,
                           NULL);
-        connectnodes (proposal->oback, proposal->origin, proposal->target);
-        connectnodes (proposalack, proposal->oback, proposal->tsister);
+        connectnodes (proposal->world,proposal->oback, proposal->origin, proposal->target);
+        connectnodes (proposal->world,proposalack, proposal->oback, proposal->tsister);
         /* either proposalack is osister or a descendent of her
            other case are disallowed by time constraints */
-        connectnodes (obb, proposal->osister, NULL);
+        connectnodes (proposal->world,obb, proposal->osister, NULL);
         adjust (proposalack, proposalack->tyme, 3);
         adjust (obb, obb->tyme, 3);
         set_dirty (proposal->oback);
@@ -375,6 +428,10 @@ void
 target_oback (proposal_fmt * proposal)
 {
     node *obb = showtop (crawlback (proposal->oback));
+#ifdef TREEDEBUG
+    printf("target_oback\n");
+#endif
+
     copy_x (proposal, proposal->xf, proposal->origin->x);
 #ifdef UEP
 
@@ -396,6 +453,10 @@ void
 target_ocousin (proposal_fmt * proposal)
 {
     node *obb;
+#ifdef TREEDEBUG
+    printf("target_ocousin\n");
+#endif
+
     obb = showtop (crawlback (proposal->oback));
     copy_x (proposal, proposal->xf, proposal->origin->x);
 #ifdef UEP
@@ -432,6 +493,9 @@ void
 target_rbcoa (proposal_fmt * proposal)
 {
     node *obb;
+#ifdef TREEDEBUG
+    printf("target_rbcoa\n");
+#endif
     if (proposal->ocousin != NULL)
     {
         obb = showtop (crawlback (proposal->oback));
@@ -476,6 +540,10 @@ void
 target_obbcoa (proposal_fmt * proposal)
 {
     node *obb, *obbb;
+#ifdef TREEDEBUG
+    printf("target_obbcoa\n");
+#endif
+
     if (proposal->ocousin != NULL)
     {
         obb = showtop (crawlback (proposal->oback));
@@ -522,6 +590,9 @@ target_stancoa (proposal_fmt * proposal)
 {
     node *obb, *nn, *oldnn, *d1, *d2, *proposalack, *uppern = NULL; /* o R */
     node **double_line;
+#ifdef TREEDEBUG
+    printf("target_stancoa\n");
+#endif
     double_line = (node **) mycalloc (2, sizeof (node *));
     if (proposal->ocousin != NULL)
     {
@@ -771,32 +842,50 @@ t_s_tcon (proposal_fmt * proposal, node * uppern)
 erase all migration nodes between proposal->origin and proposal->oback 
 */
 long
-erase_migr_nodes (node * up)
+erase_migr_nodes (world_fmt *world, node * up)
 {
     long deleted = 0;
     node *theNode, *down, *oldNode;
     down = crawlback (up);
-    theNode = up->back;
-
+    theNode = showtop(up->back);
+    //    printf("-------------\nup=%li\n",up->id);
     while (theNode->type == 'm')
     {
 
         oldNode = theNode;
-        theNode = theNode->next->back;
-        if (oldNode->type == 'm')
-        {
-            myfree(oldNode->next);
-            myfree(oldNode);
-            deleted++;
-        }
+        theNode = showtop(theNode->back);
+	free_mignodelet(oldNode,world);
+	deleted++;
+	  //printf("-");
     }
-
-
+    //printf("down=%li deleted=%li\n-------------\n",down->id, 2*deleted);
     down->back = up;
     up->back = down;
+    //if(world->heat < 0.001)
+    //  printf("%i> heat=%f deleted=%li\n",myID,1./world->heat,deleted);
     return deleted;
 }
 
+void
+erase_migr_nodes2 (world_fmt *world, node * up)
+{
+  int skip=0;
+    long deleted = 0;
+    node *theNode, *down, *oldNode;
+    if(skip!=0)
+      return;
+    down = crawlback (up);
+    theNode = showtop(up->back);
+    //    printf("-------------\nup=%li\n",up->id);
+    while (theNode->type == 'm')
+    {
+        oldNode = theNode;
+        theNode = showtop(theNode->back);
+	free_mignodelet(oldNode,world);
+	deleted++;
+	  //printf("-");
+    }
+}
 /*-------------------------------------------------------
 Connects and adjusts three nodes, the first is the 
 mother node and the next two are child nodes, if one
@@ -804,21 +893,42 @@ of the child nodes is NULL it just connects mother with
 the not-NULL child.
 */
 void
-connectnodes (node * mother, node * brother, node * sister)
+connectnodes (world_fmt *world, node * mother, node * brother, node * sister)
 {
     node *tmp;
-
+    if(mother->top!=1)
+      error("mother is not on top");
+    if(brother !=0 && brother->top!=1)
+      error("brother is not on top");
+    if(sister!=NULL && sister->top!=1)
+      error("sister is not on top");
     if ((sister != NULL) && (brother != NULL))
     {
-        if ((mother->id == brother->id) || (mother->id == sister->id)
-                || (sister->id == brother->id))
+        if ((mother == brother) || (mother == sister)
+                || (sister == brother))
         {
             error ("connectnodes() conflict");
         }
         tmp = crawl_down (brother, mother->tyme);
+#ifdef MESS
+	if(tmp->back != NULL && tmp->back->type == 'm' && showtop(tmp->back)->tyme > mother->tyme)
+	  {
+	    printf("new code 1 \n");
+	    erase_migr_nodes2(world,showtop(tmp));
+	  }
+#endif 
         mother->next->back = tmp;
         tmp->back = mother->next;
+	
         tmp = crawl_down (sister, mother->tyme);
+#ifdef MESS
+	if(tmp->back != NULL && tmp->back->type == 'm' && showtop(tmp->back)->tyme > mother->tyme)
+	  {
+	    printf("new code 2\n");
+	    erase_migr_nodes2(world,showtop(tmp));
+	  }
+#endif
+
         mother->next->next->back = tmp;
         tmp->back = mother->next->next;
     }
@@ -827,6 +937,14 @@ connectnodes (node * mother, node * brother, node * sister)
         if (sister == NULL)
         {
             tmp = crawl_down (brother, mother->tyme);
+#ifdef MESS
+	if(tmp->back != NULL && tmp->back->type == 'm' && showtop(tmp->back)->tyme > mother->tyme)
+	  {
+	    printf("new code 3\n");
+	    erase_migr_nodes2(world,showtop(tmp));
+	  }
+#endif
+
             mother->next->back = tmp;
             tmp->back = mother->next;
         }
@@ -838,12 +956,27 @@ connectnodes (node * mother, node * brother, node * sister)
                 if (mother->type == 'm')
                 {
                     tmp = crawl_down (sister, mother->tyme);
+#ifdef MESS
+		    if(tmp->back != NULL && tmp->back->type == 'm' && showtop(tmp->back)->tyme > mother->tyme)
+		      {
+			printf("new code 4 \n");
+			erase_migr_nodes2(world,showtop(tmp));
+		      }
+#endif
                     mother->next->back = tmp;
                     tmp->back = mother->next;
                 }
                 else
                 {
                     tmp = crawl_down (sister, mother->tyme);
+#ifdef MESS
+	if(tmp->back != NULL && tmp->back->type == 'm' && showtop(tmp->back)->tyme > mother->tyme)
+	  {
+	    printf("new code 5\n");
+	    erase_migr_nodes2(world,showtop(tmp));
+	  }
+#endif
+
                     mother->next->next->back = tmp;
                     tmp->back = mother->next->next;
                 }
@@ -882,7 +1015,7 @@ adjust (node * theNode, MYREAL tyme, long level)
     if (theNode->type == 'r')
     {
         theNode->v = 1;
-        theNode->length = DBL_MAX;
+        theNode->length = MYREAL_MAX;
         if (theNode->next->back != NULL)
             adjust (crawlback (theNode->next), crawlback (theNode->next)->tyme,
                     level);
@@ -992,11 +1125,11 @@ copy_x (proposal_fmt * proposal, xarray_fmt xx1, xarray_fmt xx2)
 }
 
 void
-fix_root_pop (node * p)
+fix_root_pop (world_fmt *world, node * p)
 {
     if (crawlback (p) != p->back)
     {
-        erase_migr_nodes (p);
+      erase_migr_nodes (world, p);
     }
     if (p->back->actualpop != p->actualpop)
     {
@@ -1059,13 +1192,17 @@ findcrossing (node ** ptrl1, node ** ptrl2)
 node *
 crawl_down (node * theNode, MYREAL tyme)
 {
-    node *otmp, *tmp = theNode->back;
+  node *tN = showtop(theNode);
+  node *otmp, *tmp = tN->back;
 
-    otmp = theNode;
+    otmp = tN;
     if (tmp == NULL)
         return otmp;
     while ((tmp->type == 'm') && showtop (tmp)->tyme < tyme)
     {
+#ifdef TREEDEBUG
+      printf("Crawldown: start=%x<%x> (%li<%li>)  %li %f %f\n",theNode, tN, theNode->id, tN->id,  tmp->id, showtop (tmp)->tyme, tyme);
+#endif
         otmp = tmp->next;
         tmp = tmp->next->back;
         if (tmp == NULL)
