@@ -52,7 +52,7 @@ extern float pdf_loci_histogram(world_fmt *world);
 extern void pdf_bayes_print_accept(world_fmt *world);
 // print ESS and autocorrelation
 extern void pdf_bayes_print_ess(world_fmt *world);
-extern void pdf_bayes_factor(world_fmt *world, MYREAL thermosum, long maxreplicate);
+extern void pdf_bayes_factor(world_fmt *world, MYREAL tsum, MYREAL hsum, MYREAL asum, long maxreplicate);
 extern void pdf_burnin_stops(world_fmt *world, long maxreplicate);
 // MIGHIST output
 //extern void 
@@ -78,6 +78,8 @@ extern void pdf_print_profile_table(float left_margin, float * page_height, char
 extern long prepare_profile_table(char method, long which, MYREAL *likes, MYREAL **param, world_fmt *world, long bufsize);
 extern void pdf_print_profile_percentile (world_fmt * world);
 
+extern void pdf_histogram_legend(void);
+
 // write PDF content to file
 extern int pdf_write_file(option_fmt *options);
 #endif
