@@ -16,7 +16,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: profile.c 753 2007-06-15 18:17:57Z beerli $
+$Id: profile.c 886 2007-08-18 03:09:50Z beerli $
  
 -------------------------------------------------------*/
 /*! \file profile.c
@@ -284,7 +284,7 @@ print_profile_table (char method, long which, char *var, MYREAL likes[],
     char fp[LONGLINESIZE]; //large local string buffer
     char **buffer = &world->buffer; // buffer for printing whole table
     char *temp, *var2, temp2[LINESIZE], likestr[LINESIZE], *paramstr;
-    MYREAL likemax = -DBL_MAX;
+    MYREAL likemax = -MYREAL_MAX;
 #ifdef LONGSUM
 
     elem += world->numpop * 3;
@@ -360,7 +360,7 @@ print_profile_table (char method, long which, char *var, MYREAL likes[],
     strcat (*buffer, fp);
     for (i = 0; i < GRIDSIZE; i++)
     {
-        if (likemax < MIN (likes[i], DBL_MAX))
+        if (likemax < MIN (likes[i], MYREAL_MAX))
         {
             likemax = likes[i];
             likei = i;
@@ -369,7 +369,7 @@ print_profile_table (char method, long which, char *var, MYREAL likes[],
     for (ii = 0; ii < GRIDSIZE; ii++)
     {
         i = warp (ii);
-        if (likes[i] >= DBL_MAX - EPSILON)
+        if (likes[i] >= MYREAL_MAX - EPSILON)
             sprintf (likestr, "     -    ");
         else
             sprintf (likestr, "% 6.3f%c", likes[i], likei == i ? '*' : ' ');
@@ -1214,7 +1214,7 @@ destroy_spline (spline_fmt * spline)
    and adds two additional
    points, these two points bracket the real data points:
    so that param[0]=0 and param[n+1] = 1000 * the last real param,
-   likes[0] is set -DBL_MAX and likes[n+1] is set using a linear interpolation
+   likes[0] is set -MYREAL_MAX and likes[n+1] is set using a linear interpolation
    using the last two real data points.
  
    Attention the arrays param, likes, and yy need to have a size
@@ -1522,7 +1522,7 @@ find_profilelike (MYREAL testlike, MYREAL prob, long whichprob, MYREAL minparam,
     const MYREAL probabilities[] = GRID;
     MYREAL tmp;
     MYREAL xlow, low;
-    MYREAL x, mid = -DBL_MAX;
+    MYREAL x, mid = -MYREAL_MAX;
     MYREAL xhigh, high;
     long panic = 0;
     helper_fmt helper;
