@@ -3,9 +3,7 @@
  of migration rate  and effectice population size
  using a Metropolis-Hastings Monte Carlo algorithm                            
  -------------------------------------------------------                        
- R e p o r t e r   R O U T I N E S 
- 
- reports things progress=True or verbose
+ R e p o r t e r   R O U T I N E  reports things progress=True or verbose
  
                                                                                                                Peter Beerli
  beerli@scs.fsu.edu
@@ -18,7 +16,7 @@ Copyright 2003-2007 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: reporter.c 1023 2007-11-13 05:05:42Z beerli $
+$Id: reporter.c 1105 2008-01-03 22:27:33Z beerli $
  
 -------------------------------------------------------*/
 /* \file reporter.c 
