@@ -56,10 +56,10 @@ MYREAL waiting_theta(long pop,
   
   for(i=0;i<numpop; i++)
     {
-      expected += (lineages[i] * (lineages[i]-1) * (1.0/ param[i]) +  mig0list[i] * lineages[i]) * inv_mu_rate;
+      expected += (lineages[i] * (lineages[i]-1) * (1.0/ param[i]) +  mig0list[i] * lineages[i]) ;
     }
   kpop = lineages[pop] * ( lineages[pop] -1);
-  expected = 1./interval - expected + kpop * (1.0 / param[pop])*inv_mu_rate;
+  expected = 1./interval - (expected - kpop * 1.0 / param[pop])*inv_mu_rate;
   // 
   if(expected > EPSILON)
     return kpop / expected;
@@ -106,6 +106,13 @@ void calculate_expected_values(tetra **eventbins,
 			       long numpop, 
 			       world_fmt *world)
 {
+  //  static MYREAL maxlweight = -HUGE;
+  //  MYREAL oldlweight = -HUGE;
+  boolean change;
+  //  long p;
+
+  float lweight;
+  //  float unweight;
   long i;
   long pop;
   long diff;
@@ -113,7 +120,7 @@ void calculate_expected_values(tetra **eventbins,
 
   float val;
   float se1;
-  float se2;
+  //  float se2;
 
   float weight1 = 0.0F ;
   float weight2 = 0.0F ;
@@ -121,14 +128,30 @@ void calculate_expected_values(tetra **eventbins,
   MYREAL mu_rate = world->options->mu_rates[world->locus];
   MYREAL inv_mu_rate = 1./ mu_rate;
   MYREAL inv_eventinterval = 1. / eventinterval;
-
   long firstbin = (long) ((age - interval) * inv_eventinterval);
   long bin     = (long) (age * inv_eventinterval);
-  float binage  = eventinterval * bin;
-  MYREAL firstbinage        = eventinterval * firstbin;
+  float binage  = eventinterval * bin; //this is on the left side of the bin
+  MYREAL firstbinage        = eventinterval * firstbin; //this is the age at the left side of first bin
   tetra *bins=NULL;
 
- 
+  // after discussion with Mike P. 09/28/07 DEBUG
+  //if (maxlweight >= world->likelihood[world->G])
+  //  {
+  //    lweight = EXP(world->likelihood[world->G] - maxlweight);
+  //    change = FALSE;
+  //    unweight = 1.0;
+  //  }
+  //else
+  //  {
+  //    oldlweight = maxlweight;
+  //    maxlweight = world->likelihood[world->G]; 
+  //    unweight = EXP( -maxlweight + oldlweight);
+  //    lweight = 1.0;
+  //    change = TRUE;
+  //  }
+  // enable this to disable likelihood weights
+  change =FALSE;
+  lweight = 1.0;
   // do we need to report probles when bin number is above 10000?
   // 
   if(age > 1000.0 || bin > 10000)
@@ -141,35 +164,31 @@ void calculate_expected_values(tetra **eventbins,
   if(bin < 0) 
     error("bin is smaller than zero");
 
-  diff = bin - firstbin;
-
-
   if(from == to)
     {
+      diff = bin - firstbin;
       pop  = to;
-      //      k    = lineages[pop];
-      //k   *= k-1;
-      //val  = k * interval;
       val = waiting_theta(pop,world->param0,world->mig0list, lineages,interval, inv_mu_rate, numpop); 
       if(val < 0.0)
 	return;
       //      printf("%i> k(k-1)=%li val=%f pop=%li\n",myID,k,val, pop);
       se1  = val * val;
-      se2  = se1;
+      //se2  = se1;
     } 
   else
     {
+      diff = 0;
       pop = mm2m(from, to, numpop);
       //val = lineages[to] / interval ;
       val = waiting_M(pop, to, world->param0,world->mig0list, lineages,interval, inv_mu_rate, numpop); 
       if(val < 0.0)
 	return;
       se1 = val * val;
-      se2 = 1.0;
+      //se2 = 1.0;
     }
   if(bin >= eventbinnum[pop])
     {
-      allocsize      = bin+1;
+      allocsize      = bin+10;
       eventbins[pop] = myrealloc(eventbins[pop], allocsize * sizeof(tetra));
       for(i=eventbinnum[pop];i < allocsize; i++)
 	{
@@ -181,18 +200,35 @@ void calculate_expected_values(tetra **eventbins,
 	}
       eventbinnum[pop] = allocsize;
     }
-  bins = eventbins[pop];
 
   // weights are the contribution of the estimate to the bin
   if(diff < 0)
       error("time difference is negative -- not possible");
 
   /*================================ complicated weighting*/
+  //if(change)
+  //  {
+  //    for(p=0; p < world->numpop2; p++)
+  //	{
+  //	  bins = eventbins[p];
+  //	  for(i=0; i < eventbinnum[p]; i++)
+  //	    {
+  //	      if(bins[i][0]>0.0)
+  //		{
+  //		  bins[i][0] *= unweight; 
+  //		  bins[i][1] *= unweight; 
+  //		  bins[i][2] *= unweight; 
+  //		  bins[i][3] *= unweight; 
+  //		}
+  //	    }
+  //	}
+  //  }
+  bins = eventbins[pop];
   switch(diff)
     {
       //start and stop bin are the same the whole interval does not extend over bin boundaries
     case 0: 
-      weight1 = interval * inv_eventinterval;
+      weight1 = interval * inv_eventinterval; // or =lweight=1.0
       bins[bin][0] +=  val * weight1; // parameter of interest
       bins[bin][1] += weight1;        // its weight per bin
       bins[bin][2] += se1 * weight1;  // std of parameter 
@@ -228,9 +264,9 @@ void calculate_expected_values(tetra **eventbins,
       for(i=firstbin+1; i < bin; i++)
 	{
 	  bins[i][0] += val;
-	  bins[i][1] += 1.0;
+	  bins[i][1] += lweight; // = 1.0
 	  bins[i][2] += se1;
-	  bins[i][3] += 1.0; /* 1 x 1*/
+	  bins[i][3] += lweight * lweight; /* 1 x 1*/
 	  bins[i][4] += 1.0;
 	}
       weight2 = (age - binage) * inv_eventinterval; 
@@ -240,13 +276,13 @@ void calculate_expected_values(tetra **eventbins,
       bins[bin][3] += weight2 * weight2;
       bins[bin][4] += 1.0;
     }
-  //  printf("@ %f %f %f %f\n", val, age, weight1, weight2); 
+  //  printf("@%c %li %li :%li:  %f %f %f %f %f %f %f\n", from==to ? 'c' : 'm', from, to, bin, val, age, bins[bin][0],bins[bin][1], lweight, weight1, weight2); 
 }
 
 
 ///
 /// set up the skyline plot histogram containers, only when also the migration histograms are recorded
-/// tis functiosn needs to be called AFTER the setup_mighist() function
+/// this function needs to be called AFTER the setup_mighist() function
 void
 setup_expected_events (world_fmt * world, option_fmt * options)
 {
@@ -386,19 +422,22 @@ void print_expected_values_tofile(FILE *file,  world_fmt *world)
 
   for(locus=0; locus < world->loci + sumloc; locus++)
     {
-      eventbins =  world->mighistloci[locus].eventbins;
-      eventbinnum = world->mighistloci[locus].eventbinnum;
-      eventbinsize = world->mighistloci[locus].eventbinsize;
-      for(pop = 0; pop < numpop2; pop++)
-	{  
-	  age = eventbinsize / 2.;
-	  for(i = 0; i < eventbinnum[pop]; i++)
-	    {
-		fprintf(file,"%li %li %li %10.10f %10.10f %10.10f %10.10f %10.0f\n", 
-		      locus+1, pop+1, i+1, age, eventbins[pop][i][0], eventbins[pop][i][1],eventbins[pop][i][2],eventbins[pop][i][4]);
-	      age += eventbinsize;
-	    }
-	} 
+      if(!world->data->skiploci[locus])
+	{
+	  eventbins =  world->mighistloci[locus].eventbins;
+	  eventbinnum = world->mighistloci[locus].eventbinnum;
+	  eventbinsize = world->mighistloci[locus].eventbinsize;
+	  for(pop = 0; pop < numpop2; pop++)
+	    {  
+	      age = eventbinsize / 2.;
+	      for(i = 0; i < eventbinnum[pop]; i++)
+		{
+		  fprintf(file,"%li %li %li %10.10f %10.10f %10.10f %10.10f %10.0f\n", 
+			  locus+1, pop+1, i+1, age, eventbins[pop][i][0], eventbins[pop][i][1],eventbins[pop][i][2],eventbins[pop][i][4]);
+		  age += eventbinsize;
+		}
+	    } 
+	}
     }
 }
 
@@ -421,53 +460,62 @@ void prepare_expected_values(world_fmt *world)
 
   for(locus=0; locus < world->loci; locus++)
     {
-      eventbins =  world->mighistloci[locus].eventbins;
-      eventbinnum = world->mighistloci[locus].eventbinnum;
-      for(pop = 0; pop < numpop2; pop++)
-	{  
-	  sum = (MYREAL) 0.0;
-
-	  if(eventbinnum[pop] > eventbinnum_allmax)
-	    eventbinnum_allmax = eventbinnum[pop];
-
-	  for(i = 0; i < eventbinnum[pop]; i++)
-	    {
-	      if(eventbins[pop][i][1] <= 0.0)
+      if(!world->data->skiploci[locus])
+	{
+	  eventbins =  world->mighistloci[locus].eventbins;
+	  eventbinnum = world->mighistloci[locus].eventbinnum;
+	  for(pop = 0; pop < numpop2; pop++)
+	    {  
+	      sum = (MYREAL) 0.0;
+	      
+	      if(eventbinnum[pop] > eventbinnum_allmax)
+		eventbinnum_allmax = eventbinnum[pop];
+	      
+	      for(i = 0; i < eventbinnum[pop]; i++)
 		{
-		  eventbins[pop][i][0] = 0.0;
-		  eventbins[pop][i][1] = 0.0;
-		  eventbins[pop][i][2] = 0.0;
-		  eventbins[pop][i][3] = 0.0;
-		  eventbins[pop][i][4] = 0.0;
+		  if(eventbins[pop][i][1] <= 0.0)
+		    {
+		      eventbins[pop][i][0] = 0.0;
+		      eventbins[pop][i][1] = 0.0;
+		      eventbins[pop][i][2] = 0.0;
+		      eventbins[pop][i][3] = 0.0;
+		      eventbins[pop][i][4] = 0.0;
+		    }
+		  else
+		    {
+		      eventbins[pop][i][0] /= eventbins[pop][i][1]; //average: sum(val * weight)/sum(weight)
+		      // balancing the weights sum(val^2 weights)/ sum(weights)
+		      eventbins[pop][i][2] /= eventbins[pop][i][1]; // ---> val^2
+		      // Sum(val^2) - (E(val)^2)
+		      eventbins[pop][i][2] -= (eventbins[pop][i][0] * eventbins[pop][i][0]);
+		      // 1/n (sqrt(var)
+		      //  eventbins[pop][i][2] = sqrt(fabs(eventbins[pop][i][2])/(eventbins[pop][i][4]-1.));
+		      eventbins[pop][i][2] = sqrt(fabs(eventbins[pop][i][2]));
+		      //		  eventbins[pop][i][3] = eventbins[pop][i][1];
+		    }		  
+		  sum += eventbins[pop][i][1];
 		}
-	      else
+	      for(i = 0; i < eventbinnum[pop]; i++)
 		{
-		  eventbins[pop][i][0] /= eventbins[pop][i][1];
-		  eventbins[pop][i][2] /= eventbins[pop][i][1];
-		  eventbins[pop][i][2] -= (eventbins[pop][i][0] * eventbins[pop][i][0]);
-
-		  eventbins[pop][i][2] = sqrt(fabs(eventbins[pop][i][2]));
-		  //		  eventbins[pop][i][3] = eventbins[pop][i][1];
+		  if(eventbins[pop][i][1] <= 0.0)
+		    continue;
+		  eventbins[pop][i][1] /= sum; // calculate frequency: sum(weight_i)/sum(sum(weight_i)_j)
+		  // sum(weight)/counts
+		  weight_average = eventbins[pop][i][1] / eventbins[pop][i][4];
+		  // sum(weight^2) - waverage^2
+		  eventbins[pop][i][3] -= weight_average * weight_average;
+		  eventbins[pop][i][3] = sqrt(fabs(eventbins[pop][i][3]/(eventbins[pop][i][4]-1.)));
 		}
-
-	      sum += eventbins[pop][i][1];
-	    }
-	  for(i = 0; i < eventbinnum[pop]; i++)
-	    {
-	      if(eventbins[pop][i][1] <= 0.0)
-		continue;
-	      eventbins[pop][i][1] /= sum;
-	      weight_average = eventbins[pop][i][1] / eventbins[pop][i][4];
-	      eventbins[pop][i][3] -= weight_average * weight_average;
-	      eventbins[pop][i][3] = sqrt(fabs(eventbins[pop][i][3]/(eventbins[pop][i][4]-1.)));
-	    }
-	} 
+	    } 
+	}
     }
   if(world->loci>1)
     {  
-      world->mighistloci[world->loci].eventbins = (tetra **) mycalloc(world->numpop2,sizeof(tetra *));
+      if(world->mighistloci[world->loci].eventbins == NULL)
+	world->mighistloci[world->loci].eventbins = (tetra **) mycalloc(world->numpop2,sizeof(tetra *));
+      if(world->mighistloci[world->loci].eventbinnum == NULL)
       world->mighistloci[world->loci].eventbinnum = (long *) mycalloc(world->numpop2,sizeof(long));
-      world->mighistloci[locus].eventbinsize = world->mighistloci[0].eventbinsize;
+      world->mighistloci[world->loci].eventbinsize = world->mighistloci[0].eventbinsize;
       for(pop=0; pop< world->numpop2 ; pop++)    
 	{
 	  world->mighistloci[world->loci].eventbins[pop] = (tetra *) mycalloc(eventbinnum_allmax,sizeof(tetra));
@@ -476,22 +524,25 @@ void prepare_expected_values(world_fmt *world)
       eventbins_all = world->mighistloci[world->loci].eventbins;
       for (locus = 0; locus < world->loci; locus++)
 	{
-	  eventbinnum = world->mighistloci[locus].eventbinnum;
-	  eventbins = world->mighistloci[locus].eventbins;
-	  for(pop=0; pop< world->numpop2 ; pop++)
+	  if(!world->data->skiploci[locus])
 	    {
-	      for(i=0 ; i < eventbinnum[pop]; i++)
+	      eventbinnum = world->mighistloci[locus].eventbinnum;
+	      eventbins = world->mighistloci[locus].eventbins;
+	      for(pop=0; pop< world->numpop2 ; pop++)
 		{
-		  if(eventbins[pop][i][1] > 0.0)
+		  for(i=0 ; i < eventbinnum[pop]; i++)
 		    {
-		      eventbins_all[pop][i][0] += eventbins[pop][i][0] * eventbins[pop][i][1];
-		      eventbins_all[pop][i][1] += eventbins[pop][i][1];
-		      eventbins_all[pop][i][2] += eventbins[pop][i][2] * eventbins[pop][i][1];
-		      eventbins_all[pop][i][3] += eventbins[pop][i][3] * eventbins[pop][i][1];
-		      eventbins_all[pop][i][4] += eventbins[pop][i][4];
+		      if(eventbins[pop][i][1] > 0.0)
+			{
+			  eventbins_all[pop][i][0] += eventbins[pop][i][0] * eventbins[pop][i][1];
+			  eventbins_all[pop][i][1] += eventbins[pop][i][1];
+			  eventbins_all[pop][i][2] += eventbins[pop][i][2] * eventbins[pop][i][1];
+			  eventbins_all[pop][i][3] += eventbins[pop][i][3] * eventbins[pop][i][1];
+			  eventbins_all[pop][i][4] += eventbins[pop][i][4];
+			}
+		      suml[pop] += eventbins[pop][i][1];
+		      count[pop] += eventbins[pop][i][4];
 		    }
-		  suml[pop] += eventbins[pop][i][1];
-		  count[pop] += eventbins[pop][i][4];
 		}
 	    }
 	}
@@ -501,12 +552,12 @@ void prepare_expected_values(world_fmt *world)
 	    {
 	      if(eventbins_all[pop][i][1] > 0.0)
 		{
-		  eventbins_all[pop][i][0] /= eventbins_all[pop][i][1];
-		  eventbins_all[pop][i][2] /= eventbins_all[pop][i][1];
-		  eventbins_all[pop][i][3] /= eventbins_all[pop][i][1];
-		  //eventbins_all[pop][i][0] /= suml[pop];
-		  eventbins_all[pop][i][1] /= suml[pop];
-		  //eventbins_all[pop][i][2] /= suml[pop];
+		  eventbins_all[pop][i][0] /= eventbins_all[pop][i][1];//average
+		  eventbins_all[pop][i][2] /= eventbins_all[pop][i][1];//average standard deviation
+		  eventbins_all[pop][i][3] /= eventbins_all[pop][i][1];//average weight
+		      //eventbins_all[pop][i][0] /= suml[pop];
+		      //eventbins_all[pop][i][1] /= suml[pop];
+		      //eventbins_all[pop][i][2] /= suml[pop];
 		  eventbins_all[pop][i][4] /= count[pop];
 		}
 	    }
@@ -579,19 +630,22 @@ void debug_skyline(world_fmt *world, char text[])
   fprintf(file,"# (*) values with -1 were NEVER visited\n");
   for(locus=0; locus < world->loci; locus++)
     {
-      eventbins =  world->mighistloci[locus].eventbins;
-      eventbinnum = world->mighistloci[locus].eventbinnum;
-      eventbinsize = world->mighistloci[locus].eventbinsize;
-      for(pop = 0; pop < numpop2; pop++)
-	{  
-	  age = eventbinsize / 2.;
-	  for(i = 0; i < eventbinnum[pop]; i++)
-	    {
-		fprintf(file,"%li %li %li %10.10f %10.10f %10.10f %10.10f\n", 
-		      locus+1, pop+1, i+1, age, eventbins[pop][i][0], eventbins[pop][i][1],eventbins[pop][i][0]/ eventbins[pop][i][1]);
-	      age += eventbinsize;
-	    }
-	} 
+      if(!world->data->skiploci[locus])
+	{
+	  eventbins =  world->mighistloci[locus].eventbins;
+	  eventbinnum = world->mighistloci[locus].eventbinnum;
+	  eventbinsize = world->mighistloci[locus].eventbinsize;
+	  for(pop = 0; pop < numpop2; pop++)
+	    {  
+	      age = eventbinsize / 2.;
+	      for(i = 0; i < eventbinnum[pop]; i++)
+		{
+		  fprintf(file,"%li %li %li %10.10f %10.10f %10.10f %10.10f\n", 
+			  locus+1, pop+1, i+1, age, eventbins[pop][i][0], eventbins[pop][i][1],eventbins[pop][i][0]/ eventbins[pop][i][1]);
+		  age += eventbinsize;
+		}
+	    } 
+	}
     }
   fprintf(file,"#%i >>>>>>>>>>>>>>> %s <<<<<<<<<<<<<<<<<END\n",myID,text);
 }
