@@ -14,7 +14,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: sort.c 175 2005-12-02 14:51:23Z beerli $ */
+$Id: sort.c 597 2007-02-14 15:26:53Z beerli $ */
 /* ----------------------------------------------------- */
 /*! \file sort.c */
 
@@ -204,18 +204,23 @@ intcmp (const void *v1, const void *v2)
 int
 agecmp (const void *x, const void *y)
 {
-    if (((vtlist *) x)->age < ((vtlist *) y)->age)
+  vtlist * xx = (vtlist *) x;
+  vtlist * yy = (vtlist *) y;
+  MYREAL xa = xx->age;
+  MYREAL ya = yy->age;
+
+  if (xa < ya)
     {
         return -1;
     }
     else
     {
-        if (((vtlist *) x)->age == ((vtlist *) y)->age)
+        if (xa != ya)
         {
-            return 0;
+            return 1;
         }
         else
-            return 1;
+            return 0;
     }
 }
 
